/*
  Copyright 2020 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package com.kakao.sdk.partner.auth

import com.kakao.sdk.auth.RxAuthApiClient
import com.kakao.sdk.auth.RxAuthApiClient.Companion.handleAuthError
import com.kakao.sdk.auth.model.OAuthToken
import com.kakao.sdk.auth.network.rxKauth
import com.kakao.sdk.common.network.ApiFactory
import io.reactivex.Single

private val authApi: RxPartnerAuthApi
    get() = ApiFactory.rxKauth.create(RxPartnerAuthApi::class.java)

/**
 * 그룹에 속해 있는 다른 앱의 refreshToken으로 사용자를 식별하여 신규 토큰을 발급.
 *
 * 인증에 사용된 refreshToken 값이 앱 간에 서로 공유되는 것이 아닙니다. 이 기능를 사용하면 내 앱을 위한 새로운 [OAuthToken] 세트가 발급됩니다.
 *
 * @param groupRefreshToken 사용자 인증을 위한 다른 앱의 refreshToken.
 * @return [OAuthToken]을 방출하는 [Single] 반환.
 */
fun RxAuthApiClient.issueAccessToken(groupRefreshToken: String): Single<OAuthToken> =
    authApi.issueAccessToken(
        groupRefreshToken,
        clientId = applicationInfo.appKey,
        androidKeyHash = contextInfo.signingKeyHash,
        approvalType = approvalType.value
    )
        .compose(handleAuthError())
        .map { OAuthToken.fromResponse(it) }
        .doOnSuccess { tokenManagerProvider.manager.setToken(it) }
